#include <stdint.h>
#include <stdio.h>

/* FreeRTOS */
#include "FreeRTOS.h"
#include "task.h"
#include "timers.h"

/* MX */
#include "dma.h"
#include "gpio.h"
#include "main.h"
#include "spi.h"
#include "usart.h"

/* App */
#include "app_lighting.h"
#include "app_tusb.h"

void SystemClock_Config(void);

static void app_delayed_start(TimerHandle_t tmr_handle);

int main(void) {
    /* HAL */
    HAL_Init();
    SystemClock_Config();
    MX_GPIO_Init();
    MX_LPUART1_UART_Init();
    MX_DMA_Init();
    MX_SPI1_Init();

    TimerHandle_t tmr = xTimerCreate("INIT_T", pdMS_TO_TICKS(50), pdFALSE, NULL, app_delayed_start);
    if (tmr == NULL) {
        goto dead_loop;
    }

    xTimerStart(tmr, 0);

    vTaskStartScheduler();

dead_loop:
    for (;;) {
        __WFI();
    }
}

static void app_delayed_start(TimerHandle_t tmr_handle) {
    if (app_lighting_init() < 0) {
        return;
    }

    if (app_tusb_init() < 0) {
        return;
    }
}